/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   kvdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <klocale.h>
#include "diskspace.h"
#include "settings.h"

/**
 * object initializing
 */
diskspace::diskspace(settings * prefs, int size, int free, int percent)
{
  this->prefs = prefs;
  this->size = size;
  this->free = free;
  this->percent = percent;
  this->used = size - free;
}

/**
 * simply copying an object
 */
diskspace& diskspace::operator =(diskspace diskSpace)
{
  prefs = diskSpace.prefs;
  size = diskSpace.size;
  used = diskSpace.used;
  free = diskSpace.free;
  percent = diskSpace.percent;
  return *this;
}

/**
 * destructor
 */
diskspace::~diskspace()
{
}

/**
 * check for low free space
 * @return true if free space is low
 */
bool diskspace::isLow()
{
  return percent >= prefs->spaceLowPercentFull;
}

/**
 * check for unsufficient free space
 * @param needed the needed space
 * @return true if lesser than the needed space is free
 */
bool diskspace::isFull(int needed)
{
  return needed > free;
}

/**
 * query the relative disk usage
 * @return disk usage
 */
int diskspace::getPercent()
{
  return percent;
}

/**
 * query the free space
 * @return free space in MB
 */
int diskspace::getFree()
{
  return free;
}

/**
 * query the disk usage in printable form
 * @return string containing some infos about the disk usage
 */
QString diskspace::toString(int needed)
{
  char buffer[100];
  QString text("<ul>");
  sprintf(buffer, "%.1f GB", (double)(size / 1024.0));
  text.append("<li>").append(i18n("Size")).append(": ").append(buffer).append("</li>");

  sprintf(buffer, "%.1f GB (%d %%)", (double)(free / 1024.0), 100-percent);
  text.append("<li>").append(i18n("Free")).append(": ").append(buffer).append("</li>");

  sprintf(buffer, "%.1f GB", (double)(needed / 1024.0));
  text.append("<li>").append(i18n("Needed")).append(": ").append(buffer).append("</li>");

  text.append("</ul>");

  return text;
}
