/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   vdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <klocale.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <krestrictedline.h>
#include <kcombobox.h>
#include <knuminput.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qgridlayout.h>
#include <qframe.h>

#include "preferences.h"
#include "timeselector.h"
#include "fileselector.h"

preferences::preferences(QWidget * parent) : KPageDialog(parent)
{
  // style
  setFaceType(KPageDialog::Tabbed);
  setModal(true);
  setCaption(i18n("Configure Vdrmon"));
  setButtons(KDialog::Ok|KDialog::Cancel);

  // tab pages
  createVdrPage();
  createShutdownPage();
  createSoundPage();
  createTrayPage();
  createDiskPage();
}


preferences::~preferences()
{
}

void preferences::createVdrPage()
{
  // page
  QFrame * page = new QFrame(this);
  page->setObjectName("vdrpage");
  addPage(page, i18n("VDR"));

  // host
  QLabel * labelHost = new QLabel(i18n("Host:"), page);
  KRestrictedLine * editHost = new KRestrictedLine(page);
  editHost->setObjectName("kcfg_VdrHost");

  // port
  QLabel * labelPort = new QLabel(i18n("Port:"), page);
  KIntNumInput * numPort = new KIntNumInput(page);
  numPort->setObjectName("kcfg_VdrPort");
  numPort->setRange(1, 65535);

  // alive messages
  QCheckBox * checkAlive = new QCheckBox(i18n("Send alive messages"), page);
  checkAlive->setObjectName("kcfg_SendAlive");
  QLabel * labelAlive = new QLabel(i18n("Interval:"), page);
  KIntNumInput * numAlive = new KIntNumInput(page);
  numAlive->setObjectName( "kcfg_AliveInterval");
  numAlive->setRange(1, 65535);

  QLabel * unitAlive = new QLabel(i18n("second(s)"), page);
  QLabel * labelTimeout = new QLabel(i18n("Timeout:"), page);
  numTimeout = new KIntNumInput(page);
  numTimeout->setObjectName("kcfg_AliveTimeout");
  numTimeout->setRange(10, 3610);
  QLabel * unitTimeout = new QLabel(i18n("second(s)"), page);

  // enable settings if no alive messages
  connect(checkAlive, SIGNAL(toggled(bool)), labelAlive, SLOT(setEnabled(bool)));
  connect(checkAlive, SIGNAL(toggled(bool)), numAlive, SLOT(setEnabled(bool)));
  connect(checkAlive, SIGNAL(toggled(bool)), unitAlive, SLOT(setEnabled(bool)));
  connect(checkAlive, SIGNAL(toggled(bool)), labelTimeout, SLOT(setEnabled(bool)));
  connect(checkAlive, SIGNAL(toggled(bool)), numTimeout, SLOT(setEnabled(bool)));
  connect(checkAlive, SIGNAL(toggled(bool)), unitTimeout, SLOT(setEnabled(bool)));
  checkAlive->setChecked(true);

  // timeout must be lesser than interval
  connect(numAlive, SIGNAL(valueChanged(int)), this, SLOT(aliveChanged(int)));

  // layout
  QGridLayout * grid = new QGridLayout(page);
  grid->setObjectName("vdrpage-layout");

  QGridLayout * ipGrid = new QGridLayout();
  ipGrid->setObjectName("vdrpage-ip-layout");
  ipGrid->setColumnStretch(2, 1);
  ipGrid->addWidget(labelHost, 0, 0, 1, 1); // ok
  ipGrid->addWidget(editHost, 0, 1, 1, 2); // ok
  ipGrid->addWidget(labelPort, 1, 0, 1, 1); // ok
  ipGrid->addWidget(numPort, 1, 1, 1, 1); // ok

  QGridLayout * aliveGrid = new QGridLayout();
  aliveGrid->setObjectName("vdrpage-alive-layout");
//  aliveGrid->setColSpacing(0, 40);
  aliveGrid->setColumnStretch(3, 1);
  aliveGrid->addWidget(checkAlive, 0, 0, 1, 4); // ok
  aliveGrid->addWidget(labelAlive, 1, 1, 1, 1); // ok
  aliveGrid->addWidget(numAlive, 1, 2, 1, 1); // ok
  aliveGrid->addWidget(unitAlive, 1, 3, 1, 1); // ok
  aliveGrid->addWidget(labelTimeout, 2, 1, 1, 1); // ok
  aliveGrid->addWidget(numTimeout, 2, 2, 1, 1); // ok
  aliveGrid->addWidget(unitTimeout, 2, 3, 1, 1); // ok

  grid->addLayout(ipGrid, 0, 0);
  grid->addLayout(aliveGrid, 1, 0);
}

void preferences::aliveChanged(int value)
{
  int min = value + 10;

  // adjust minValue for timeout
  numTimeout->setMinimum(min);

  // adjust value if too small
  if (numTimeout->value() < min)
    numTimeout->setValue(min);
}

void preferences::createShutdownPage()
{
  // page
  QFrame * page = new QFrame(this);
  page->setObjectName("shutdownpage");
  addPage(page, i18n("Shutdown"));

  // checkboxes
  QCheckBox * checkRecording = new QCheckBox(i18n("Warn if shutdown while recording"), page);
  checkRecording->setObjectName("kcfg_WarnRecording");
  QCheckBox * checkScheduled = new QCheckBox(i18n("Warn if shutdown while scheduled recording"), page);
  checkScheduled->setObjectName("kcfg_WarnScheduled");
  timeselector * warnTime = new timeselector(page, i18n("during the next"), "WarnScheduledTime");
  QLabel * label = new QLabel(i18n("Wait before shutdown"), page);
  KIntNumInput * shutdownWait = new KIntNumInput(page);
  shutdownWait->setObjectName("kcfg_ShutdownWait");
  shutdownWait->setRange(0, 36000);
  QLabel * unit = new QLabel(i18n("seconds"), page);

  // layout
  QGridLayout * gridWait = new QGridLayout();
  gridWait->setObjectName("shutdownpage-wait-layout");
  gridWait->setColumnStretch(2, 1);
  gridWait->addWidget(label, 0, 0, 1, 1); // ok
  gridWait->addWidget(shutdownWait, 0, 1, 1, 1); // ok
  gridWait->addWidget(unit, 0, 2, 1, 1); // ok

  QGridLayout * grid = new QGridLayout(page);
  grid->setObjectName("shutdownpage-layout");
//  grid->setColSpacing(0, 40);
  grid->setColumnStretch(2, 1);
  grid->setRowStretch(4, 1);
  grid->addWidget(checkRecording, 0, 0, 1, 3); // ok
  grid->addWidget(checkScheduled, 1, 0, 1, 4);; // ok
  grid->addWidget(warnTime, 2, 1, 1, 1); // ok

  grid->addLayout(gridWait, 3, 0, 1, 3); // ok


  // slots
  connect(checkScheduled, SIGNAL(toggled(bool)), warnTime, SLOT(setEnabled(bool)));

  // init
  checkRecording->setChecked(true);
  checkScheduled->setChecked(true);
}

void preferences::createSoundPage()
{
  // page
  QFrame * page = new QFrame(this);
  page->setObjectName("soundpage");
  addPage(page, i18n("Recordings"));

  // start file
  fileselector * start = new fileselector(page, i18n("Sound when recording starts"),
                                          i18n("Sound file:"), "StartSound", false);
  fileselector * stop = new fileselector(page, i18n("Sound when recording ends"),
                                         i18n("Sound file:"), "StopSound", false);

  // grid
  QGridLayout * grid = new QGridLayout(page);
  grid->setObjectName("soundpage-layout");
//  grid->setColSpacing(0, 40);
  grid->setColumnStretch(2, 1);
  grid->setRowStretch(4, 1);
  grid->addWidget(start, 0, 0, 1, 3); // ok
  grid->addWidget(stop, 1, 0, 1, 3); // ok
}

void preferences::createTrayPage()
{
  QFrame * page = new QFrame(this);
  page->setObjectName("traypage");
  addPage(page, i18n("Tray"));

  // checkboxes
  QCheckBox * checkTimer = new QCheckBox(i18n("Show timer"), page);
  checkTimer->setObjectName("kcfg_ShowTimer");

  QCheckBox * checkToggle = new QCheckBox(i18n("Toggle status and timer"), page);
  checkToggle->setObjectName("kcfg_ToggleTimer");

  KComboBox * timerMode = new KComboBox(page);
  timerMode->setObjectName("kcfg_TimerMode");
  timerMode->addItem(i18n("Countdown to next recording"));
  timerMode->addItem(i18n("Time of the next recording"));

  QCheckBox * checkToggleTimerIfIdle = new QCheckBox(i18n("Don't toggle timer if idle"), page);
  checkToggleTimerIfIdle->setObjectName("kcfg_DontToggleIfIdle");

  QCheckBox * checkEventPopup = new QCheckBox(i18n("Show popup on events"), page);
  checkEventPopup->setObjectName("kcfg_ShowEventPopup");

  QLabel * labelTime = new QLabel(i18n("Toggle every"), page);
  KIntNumInput * toggleTimerInterval = new KIntNumInput(page);
  toggleTimerInterval->setObjectName("kcfg_ToggleTimerInterval");
  toggleTimerInterval->setRange(1, 10);
  QLabel * labelUnit = new QLabel(i18n("second(s)"), page);

  QCheckBox * checkSpace = new QCheckBox(i18n("Show disk usage"), page);
  checkSpace->setObjectName("kcfg_ShowSpace");
  KComboBox * spaceMode = new KComboBox(page);
  spaceMode->setObjectName("kcfg_SpaceMode");
  spaceMode->addItem(i18n("Always"));
  spaceMode->addItem(i18n("If not enough space"));
  spaceMode->addItem(i18n("If space is low"));

  // grid
  QGridLayout * grid = new QGridLayout(page);
  grid->setObjectName("traypage-layout");
//  grid->setColSpacing(0, 40);
  grid->setColumnStretch(4, 1);
  grid->addWidget(checkTimer, 0, 0, 1, 3); // ok
  grid->addWidget(timerMode, 0, 3, 1, 2); // ok
  grid->addWidget(checkToggle, 1, 0, 1, 5); // ok
  grid->addWidget(checkToggleTimerIfIdle, 2, 0, 1, 5); // ok
  grid->addWidget(checkEventPopup, 3, 0, 1, 5); // ok
  grid->addWidget(labelTime, 4, 0, 1, 1); // ok
  grid->addWidget(toggleTimerInterval, 4, 1, 1, 1); // ok
  grid->addWidget(labelUnit, 4, 2, 1, 3); // ok
  grid->addWidget(checkSpace, 5, 0, 1, 3); // ok
  grid->addWidget(spaceMode, 5, 3, 1, 2); // ok

  // slots
  connect(checkTimer, SIGNAL(toggled(bool)), timerMode, SLOT(setEnabled(bool)));
  connect(checkToggle, SIGNAL(toggled(bool)), labelTime, SLOT(setEnabled(bool)));
  connect(checkToggle, SIGNAL(toggled(bool)), toggleTimerInterval, SLOT(setEnabled(bool)));
  connect(checkToggle, SIGNAL(toggled(bool)), labelUnit, SLOT(setEnabled(bool)));
  connect(checkToggle, SIGNAL(toggled(bool)), checkToggleTimerIfIdle, SLOT(setEnabled(bool)));

  // init
  checkTimer->setChecked(true);
}

void preferences::createDiskPage()
{
  QFrame * page = new QFrame(this);
  page->setObjectName("diskpage");
  addPage(page, i18n("Space"));

  // disk check
  QCheckBox * checkSpace = new QCheckBox(i18n("Monitor free disk space"), page);
  checkSpace->setObjectName("kcfg_SpaceMonitor");
  QLabel * labelSpace = new QLabel(i18n("Space per hour (MB):"), page);
  KIntNumInput * numSpace = new KIntNumInput(page);
  numSpace->setObjectName("kcfg_MBPerHour");
  numSpace->setRange(2000, 20000);
  QLabel * labelLow = new QLabel(i18n("Signal low space if disk is filled more than (%):"), page);
  KIntNumInput * numLow = new KIntNumInput(page);
  numLow->setObjectName("kcfg_LowPercentFull");
  numLow->setRange(1, 100);
  fileselector * noSpaceSound = new fileselector(page, i18n("Sound when not enough disk space"),
                                              i18n("Sound file:"), "NoSpaceSound", false);
  fileselector * lowSpaceSound = new fileselector(page, i18n("Sound when disk space is low"),
                                              i18n("Sound file:"), "LowSpaceSound", false);

  // grid
  QGridLayout * grid = new QGridLayout(page);
  grid->setObjectName("diskpage-layout");
  grid->setColumnStretch(2, 1);
  grid->addWidget(checkSpace, 0, 0, 1, 3); // ok
  grid->addWidget(labelSpace, 1, 0, 1, 1); // ok
  grid->addWidget(numSpace, 1, 1, 1, 1); // ok
  grid->addWidget(labelLow, 2, 0, 1, 1); // ok
  grid->addWidget(numLow, 2, 1, 1, 1); // ok
  grid->addWidget(noSpaceSound, 3, 0, 1, 3); // ok
  grid->addWidget(lowSpaceSound, 4, 0, 1, 3); // ok
}
