/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   vdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdio.h>
#include <qstringlist.h>
#include <klocale.h>

#include "recording.h"


recording::recording()
{
}

recording::recording(const QString& text)
{
  // get duration
  QStringList columns = text.split(' ');
  startTime = columns[0].toULong(0, 16);
  stopTime = columns[1].toULong(0, 16);
  columns = text.split(':');
  channel = columns[1];
  start = columns[3].left(2) + ":" + columns[3].right(2);
  stop = columns[4].left(2) + ":" + columns[4].right(2);
  singleLineTitle = columns[7];

  // resamble splitted comment
  comment = "";
  for (int i = 8; i < columns.size()-1; i++)
  {
    comment += columns[i] + ":";
  }
  comment += columns[columns.size()-1];

  title = splitLines(singleLineTitle);
  comment = splitLines(comment);
}

QString recording::splitLines(QString text) const
{
  QStringList words = text.replace("|", " \n ").split(' ');
  QString line;
  QString lines;

  for (int i = 0; i < words.size(); i++)
  {
    if (words[i] == "\n")
    {
      lines += line + "\n\n";
      line = "";
    }
    else if (line.length() < 30)
    {
      line += words[i] + " ";
    }
    else
    {
      lines += line + "\n";
      line = "";
      i--;
    }
  }
  lines += line;

  return lines;
}

recording::~recording()
{
}

QString recording::getStart() const
{
  return start;
}

QString recording::getStop() const
{
  return stop;
}

QString recording::getTitle() const
{
  return title;
}

QString recording::getSingleLineTitle() const
{
  return singleLineTitle;
}

QString recording::getComment() const
{
  return comment;
}

double recording::getHours() const
{
  return hours;
}

int recording::getRemainingSeconds() const
{
  time_t now = time(0);
  if (startTime > now)
    return stopTime - startTime;
  else
    return stopTime - now;
}

QString recording::getRemaining() const
{
  int rest = getRemainingSeconds();
  char restText[10];

  sprintf(restText, "%02d:%02d", rest / 3600, (rest / 60) % 60);

  return (startTime > time(0) ? i18n("Duration") : i18n("Remaining")) + restText;
}

int recording::getNeededSpace(int MBperHour) const
{
  return (int)((getRemainingSeconds() / 3600.0) * MBperHour + 0.5);
}

bool recording::operator ==(const recording& rec) const
{
  return start == rec.start && stop == rec.stop && channel == rec.channel;
}

QString recording::toString() const
{
  return getSingleLineTitle() + " (" + getRemaining() + ")";
}

