/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   kvdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <klocale.h>
#include <kapplication.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>
#include <QFrame>
#include <QGridLayout>
#include <kprogressdialog.h>
#include <kworkspace/kworkspace.h>

#include "shutdown.h"

shutdown::shutdown(KApplication * app)
  : KDialog()
	// Plain, Qt::WType_TopLevel | Qt::WStyle_Customize | Qt::WStyle_StaysOnTop | Qt::WStyle_DialogBorder, NULL, NULL, false,
          //      i18n("Computer shutdown"), Cancel, Cancel)
{
  // layout
  setCaption(i18n("Computer shutdown"));

  // save application
  this->app = app;

  // get page
  QFrame * page = new QFrame(this);
  setMainWidget(page);

  // add controls
  QLabel * labelText = new QLabel(i18n("Kvdrmon wants to shutdown your computer. Press CANCEL to abort!"), page);
  progress = new QProgressBar(page);
  progress->setTextVisible(false);

  // layout
  QGridLayout * layout = new QGridLayout(page);
  layout->setColumnStretch(3, 1);
  layout->addWidget(labelText, 0, 0);
  layout->addWidget(progress, 1, 0);

  // timer
  timer = new QTimer(this);

  // signals
  connect(this, SIGNAL(cancelClicked()), this, SLOT(cancelClicked()));
  connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
}

shutdown::~shutdown()
{
}

void shutdown::exec(int wait)
{
  remaining = wait;
  timer->start(1000);

  progress->setMaximum(wait);
  progress->setValue(wait);

  KDialog::exec();
}

void shutdown::cancelClicked()
{
  // no shutdown
  timer->stop();
  close();
}

void shutdown::timeout()
{
  if (--remaining == 0)
  {
    // stop timer
    timer->stop();
    close();
    // shutdown
    KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmNo, KWorkSpace::ShutdownTypeHalt, KWorkSpace::ShutdownModeTryNow);
  } else {
    progress->setValue(remaining);
  }
}
